  /*
  Modified version of JohnSL's FSR Trigger for use on the Trinket or ATtiny85
  From: https://github.com/ianlee74/AutoTuningFSRTrigger/blob/master/JohnSL_Trigger_Rewritten_For_Trinket_usingInternalPullUps/JohnSL_Trigger_Rewritten_For_Trinket_usingInternalPullUps.ino
  And from: http://trains.socha.com/2014/07/updating-fsr-endstop-controller-firmware.html

  Loses the following features from his design:
  * Loses LED indicators per FSR
  * Loses Various other IO pins
  
  Retains the following:
  * 1 FSR input (wire the three FSR's in parallel across Trinket pin #2 and GND)
  * JohnSL's long and short averaging routines
  
  Gains:
  * the ability to re-program via the USB since pins 3 and 4 are shared with the USB
 
  */
 
 /* 
  Trinket Pin Assignments:
  
  Bat+ - +5 from the RAMPS ZMin pins
  GND  - Ground from the RAMPS Zmin pins, FSRs
  #4   - N/C
  #3   - N/C
  Rst  - N/C
  USB+ - N/C
  #0   - Signal to the RAMPS Zmin pins
  #1   - N/C  (This is used for the trigger on board LED)
  #2   - FSRs  (This is A/D input #1)
  5V   - N/C 
  
  * Changed the LED output pin to pin 1
  * Changed endstop signal output to pin 0
  * Changed the ADC chosen for the FSRs to be A1 (pin 2)

  */
  
// Define the pin that has an LED on it. We have an LED for the FSR to indicate
// when the FSR is triggered. And one power/end stop LED that is on until an
// FSR is triggered.
#define LEDTRIGGER  01
#define ENDSTOP     00

short fsrPin = 2;               // The on-board Trinket pin number for the FSR analog input
short fsrAnalogNum = 1;         // associated analog pin number to use with analogRead()

#define SHORT_SIZE 8
#define LONG_SIZE 16
#define LONG_INTERVAL (2000 / LONG_SIZE)

unsigned long lastLongTime;              // Last time in millis that we captured a long-term sample
uint16_t longSamples[LONG_SIZE];         // Used to keep a long-term average
uint8_t longIndex = 0;                   // Index of the last long-term sample
uint16_t longAverage = 0;

uint16_t shortSamples[SHORT_SIZE];       // Used to create an average of the most recent samples
uint8_t averageIndex = 0;

uint8_t pin=0;                           // Defining a global pin variable that is used in the code.

void SetOutput(bool triggered)
{
    // Turns on the FSR LED when the FSR is triggered.
    digitalWrite(LEDTRIGGER, triggered ? HIGH : LOW);
    // Send the signal for the Z-probe. 
    digitalWrite(ENDSTOP, triggered ? HIGH : LOW);  // reverse high and low here for a Normally closed expectation
}

void InitValues()
{
  for (uint8_t i = 0; i < SHORT_SIZE; i++)
    shortSamples[i] = 0;

  for (uint8_t i = 0; i < LONG_SIZE; i++)
    longSamples[i] = 0;

  lastLongTime = millis();
}

//
// One-time setup for the various I/O ports
//
void setup()
{
  InitValues();

  pinMode(fsrPin, INPUT);      // init the A/D converter for use with the FSRs
  digitalWrite(fsrPin, HIGH);  // Use the internal pull-up resistors.

  // the green onboard LED acts as a power-on indicator.
  // The red onboard LED will flash whenever the device triggers
  pinMode(LEDTRIGGER, OUTPUT);
  digitalWrite(LEDTRIGGER, HIGH);

  // Set the endstop pin to be an output that is set for Normally Open switching
  pinMode(ENDSTOP, OUTPUT);
  digitalWrite(ENDSTOP, LOW );
};

//
// Captures a new value once LONG_INTERVAL ms have passed since the last sample.
//
// Returns: The current long-range average
uint16_t UpdateLongSamples(int avg)
{
  // If enough time hasn't passed, just return the last value.
  unsigned long current = millis();
  if (current - lastLongTime <= LONG_INTERVAL)
    return longAverage;

  // Update the long sample with the new value, and then update the long average.
  longSamples[longIndex++] = avg;
  if (longIndex >= LONG_SIZE)
      longIndex = 0;

  uint16_t total = 0;
  for (int i = 0; i < LONG_SIZE; i++)
    total += longSamples[i];
    
  longAverage = total / LONG_SIZE;

  lastLongTime = millis();
  return longAverage;
}

void CheckIfTriggered()
{
  // Calculate the average of the most recent short-term samples.
  uint16_t total = 0;
  for (int i = 0; i < SHORT_SIZE; i++)
  {
    total += shortSamples[i];
  }
  uint16_t avg = total / SHORT_SIZE;
  uint16_t longAverage = UpdateLongSamples(avg);
  uint16_t threshold = 0.88 * longAverage;   // factor to reduce/increase sensitivity
  bool triggered = avg < threshold;
  SetOutput(triggered);
}

void loop()
{
  int value = analogRead(fsrAnalogNum);
  shortSamples[averageIndex++] = value;
  if (averageIndex >= SHORT_SIZE)
    averageIndex = 0;
  CheckIfTriggered();
};
